import pytest
import jetkiller


def test_convert_file_1():
    """2 args, no colormap."""
    jetkiller.convert_file("tests/test_data/test_input_1.png", "tests/test_data/test_result_1.png")
    assert True


def test_convert_file_2():
    """2 args, no colormap."""
    jetkiller.convert_file("tests/test_data/test_input_2.png", "tests/test_data/test_result_2.png")
    assert True


def test_convert_file_3():
    """1 arg, no colormap."""
    jetkiller.convert_file("tests/test_data/test_input_1.png")
    assert True


def test_convert_file_4():
    """1 arg, wrong colormap."""
    with pytest.raises(Exception) as e:
        jetkiller.convert_file("tests/test_data/test_input_1.png", colormap="blabla")
    assert e.type == ValueError


def test_convert_file_5():
    """1 arg, other colormap."""
    jetkiller.convert_file("tests/test_data/test_input_1.png", colormap="magma")
    assert True


def test_convert_file_6():
    """Protected output."""
    with pytest.raises(Exception) as e:
        jetkiller.convert_file("tests/test_data/test_input_1.png", "tests/test_data/protected_output_file.png")
    assert e.type == PermissionError


def test_convert_file_7():
    """Wrong input type"""
    with pytest.raises(Exception):
        jetkiller.convert_file("tests/test_data/generate_test_images.py", "tests/test_data/protected_output_file.png")


def test_convert_file_8():
    """File not found."""
    with pytest.raises(Exception) as e:
        jetkiller.convert_file("tests/test_data/not_existing.png", "tests/test_data/protected_output_file.png")
    assert e.type == FileNotFoundError



import sys
import argparse
import jetkiller.file as jkfi
import jetkiller.config as cfg
import jetkiller.gui as gui


def parse_args(argv):
    """Parse command line arguments."""
    parser = argparse.ArgumentParser()
    parser.add_argument("-g", "--gui", action='store_true')
    parser.add_argument("input_file", nargs="?", default=None)
    parser.add_argument("output_file", nargs="?", default=None)
    parser.add_argument("-cm", "--colormap", type=str, default=cfg.default_output_colormap)
    args = parser.parse_args(argv)
    return args


def main_cl(args):
    """Entry-point of the command-line interface"""
    try:
        jkfi.convert_file(args.input_file, args.output_file, args.colormap)
    except Exception as e:
        # Abort on errors
        print(e, file=sys.stderr)
        exit(type(e).__name__)


def main_gui():
    """Entry-point of the graphical user interface."""
    gui.main()


def main(argv=None):
    """Main entry-point."""
    args = parse_args(argv)
    if args.gui:
        main_gui()
    else:
        main_cl(args)


if __name__ == "__main__":
    main()



import numpy as np
import pytest
import jetkiller.image as jkim
from PIL import Image


@pytest.fixture
def data_array():
    return np.array([[[128, 128, 128], [4, 5, 6]],
                     [[128, 128, 128], [4, 5, 6]],
                     [[7, 8, 9], [10, 11, 12]],
                     [[254, 254, 255], [255, 255, 255]]])


@pytest.fixture
def image():
    return Image.new("RGB", (10, 11))


def test_convert_image_1(image):
    assert isinstance(jkim.convert_image(image), Image.Image)


def test_convert_image_2(image):
    assert image.size == jkim.convert_image(image).size


def test_array2image_1(data_array):
    assert isinstance(jkim.array2image(data_array), Image.Image)


def test_array2image_2(data_array):
    w, h = jkim.array2image(data_array).size
    d1, d2, _ = data_array.shape
    assert w == d2 and h == d1


def test_image2array_1(image):
    assert isinstance(jkim.image2array(image), np.ndarray)


def test_image2array_2(image):
    w, h = image.size
    d1, d2, _ = jkim.image2array(image).shape
    assert w == d2 and h == d1



import numpy as np
import jetkiller.array as jkar
import jetkiller.config as cfg
import pytest


@pytest.fixture
def data_array():
    return np.array([[[128, 128, 128], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]], [[254, 254, 255], [255, 255, 255]]])


def test_get_colormap_1():
    assert jkar.get_colormap("viridis").shape == (jkar._colormap_size, len(cfg.mode))


def test_get_colormap_2():
    assert jkar.get_colormap("jet").shape == (jkar._colormap_size, len(cfg.mode))


def test_get_colormap_3():
    assert 0 <= np.max(jkar.get_colormap("viridis")) <= jkar._nint


def test_get_colormap_4():
    assert 0 <= np.max(jkar.get_colormap("jet")) <= jkar._nint


def test_get_colormap_5():
    assert (jkar.get_colormap("viridis") % 1 == 0).all()


def test_get_colormap_6():
    assert (jkar.get_colormap("jet") % 1 == 0).all()


def test_convert_array_1(data_array):
    assert data_array.shape == jkar.convert_array(data_array).shape


def test_convert_array_2(data_array):
    assert (jkar.convert_array(data_array) % 1 == 0).all()


def test_convert_array_3(data_array):
    cmap = jkar.get_colormap("viridis")
    dim1 = range(data_array.shape[0])
    dim2 = range(data_array.shape[1])
    data = jkar.convert_array(data_array, "viridis")
    for i in dim1:
        for j in dim2:
            assert (data[i, j] in cmap) or (data[i, j, 0] == data[i, j, 1] and data[i, j, 1] == data[i, j, 2])



from PIL import Image
import numpy as np
import jetkiller.config as cfg
import jetkiller.array as jkar


def convert_image(image, colormap=cfg.default_output_colormap):
    """Convert an image from the "jet colormap" to a better one."""
    data = image2array(image)
    data = jkar.convert_array(data, colormap)
    output = array2image(data)
    return output


def array2image(array):
    """Convert an array to an image."""
    mode = "RGB" if array.shape[2] == 3 else "RGBA"
    return Image.fromarray(array, mode=mode)


def image2array(image):
    """Convert an image to an array."""
    data = np.copy(np.asarray(image))
    return data



from PIL import Image
import jetkiller.image as jkim
import jetkiller.config as cfg
from pathlib import Path


def convert_file(input_filename, output_filename=None, colormap=cfg.default_output_colormap):
    """Convert an image file from the "jet" colormap to a better one."""
    input_image_data = read_image(input_filename)
    output_image_data = jkim.convert_image(input_image_data, colormap)
    if output_filename is None:
        input_path = Path(input_filename)
        output_basename = input_path.stem + cfg.default_output_suffix + input_path.suffix
        output_filename = str(input_path.parent / output_basename)
    write_image(output_filename, output_image_data)


def read_image(input_filename):
    """Read an image from a file."""
    image = Image.open(input_filename)
    image_converted = image.convert(mode=cfg.mode)
    return image_converted


def write_image(output_filename, image):
    """Save an image to a file."""
    image.save(output_filename)



import pytest
from jetkiller import __main__


def test_main_1():
    """No args."""
    with pytest.raises(SystemExit) as e:
        argv = None
        __main__.main(argv)
    assert e.type == SystemExit


def test_main_2():
    """2 args, no colormap."""
    argv = ["tests/test_data/test_input_1.png", "tests/test_data/test_result_1.png"]
    __main__.main(argv)
    assert True


def test_main_3a():
    """Only one argument"""
    argv = ["tests/test_data/test_input_1.png"]
    __main__.main(argv)
    assert True


def test_main_3b():
    """Only one argument"""
    argv = ["tests/test_data/test_input_2.png"]
    __main__.main(argv)
    assert True


def test_main_4():
    """Wrong argument."""
    with pytest.raises(SystemExit) as e:
        argv = ["tests/test_data/test_input_1.png", "--bullshit"]
        __main__.main(argv)
    assert e.type == SystemExit
    assert e.value.code == 2


def test_main_5():
    """Wrong colormap."""
    with pytest.raises(SystemExit) as e:
        argv = ["tests/test_data/test_input_1.png", "--colormap", "blabla"]
        __main__.main(argv)
    assert e.type == SystemExit
    assert e.value.code == "ValueError"


def test_main_6():
    """No colormap."""
    with pytest.raises(SystemExit) as e:
        argv = ["tests/test_data/test_input_1.png", "--colormap"]
        __main__.main(argv)
    assert e.type == SystemExit
    assert e.value.code == 2


def test_main_7():
    """Other colormap."""
    argv = ["tests/test_data/test_input_1.png", "tests/test_data/test_result_magma.png", "--colormap", "magma"]
    __main__.main(argv)
    assert True


def test_main_8():
    """Protected output."""
    with pytest.raises(SystemExit) as e:
        argv = ["tests/test_data/test_input_1.png", "tests/test_data/protected_output_file.png"]
        __main__.main(argv)
    assert e.type == SystemExit
    assert e.value.code == "PermissionError"


def test_main_9():
    """Wrong input type"""
    with pytest.raises(SystemExit) as e:
        argv = ["tests/test_data/generate_test_images.py", "tests/test_data/wrong_type.png"]
        __main__.main(argv)
    assert e.type == SystemExit


def test_main_10():
    """File not found."""
    with pytest.raises(SystemExit) as e:
        argv = ["tests/test_data/not_existing.png", "tests/test_data/wrong_type.png"]
        __main__.main(argv)
    assert e.type == SystemExit
    assert e.value.code == "FileNotFoundError"



import numpy as np
from matplotlib import cm
import functools
import jetkiller.config as cfg

# Internal package parameters
_colormap_size = 512
_nint = 255
_type = float
_cache_size = 1024


def get_colormap(colormap):
    """Return a colormap as an array. `colormap` is any name recognized by matplotlib."""
    min_val = 0
    max_val = 1
    cmap = cm.get_cmap(colormap, _colormap_size)
    data = np.linspace(min_val, max_val, _colormap_size)
    color_table = np.around(cmap(data) * _nint)
    return np.array(color_table[:, 0:len(cfg.mode)], dtype=_type)


def convert_array(data, colormap=cfg.default_output_colormap):
    """Convert an image array from the "jet" colormap to a better one."""

    input_cmap = get_colormap(cfg.default_input_colormap)
    output_cmap = get_colormap(colormap)

    @functools.lru_cache(maxsize=_cache_size)
    def convert_pixel(red, green, blue):
        # Get nearest color from input colormap and return
        # corresponding color in output colormap
        dr = input_cmap[:, 0] - red
        dg = input_cmap[:, 1] - green
        db = input_cmap[:, 2] - blue
        dist = dr * dr + dg * dg + db * db
        idx = np.argmin(dist)
        return output_cmap[idx]

    dim1 = range(data.shape[0])
    dim2 = range(data.shape[1])
    for i in dim1:
        for j in dim2:
            r, g, b = data[i, j, 0], data[i, j, 1], data[i, j, 2]
            if r != g or g != b:  # Grey pixels are not processed
                data[i, j, 0:3] = convert_pixel(r, g, b)
    return data



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jetkiller",
    version="1.0.1",
    author="Arnaud-D",
    description="Enhance data visualizations using the \"jet\" colormap.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Arnaud-D/jetkiller",
    packages=setuptools.find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=["pillow", "numpy", "matplotlib"],
    entry_points={
        'console_scripts': [
            'jetkiller = jetkiller.__main__:main',
        ],
    }
)



import tkinter as tk
import tkinter.filedialog as fd
import jetkiller as jk
from PIL import Image
import PIL.ImageTk as PilTk
import sys


def scaling_transform(w1, h1, w2, h2):
    """Rescale rectangle (w1, h1) to fit in rectangle (w2, h2) without changing the ratio."""
    r1 = w1 / h1
    r2 = w2 / h2
    if r1 <= r2:
        h = h2
        w = r1 * h2
    else:
        w = w2
        h = w2 / r1
    return w, h


def update_preview(image, preview_canvas):
    """Use an image to update a preview canvas."""
    frame_width, frame_height = int(preview_canvas["width"]), int(preview_canvas["height"])
    w1, h1 = image.width, image.height
    w, h = scaling_transform(w1, h1, frame_width, frame_height)
    image_miniature = image.resize((int(w), int(h)))
    tk_miniature = PilTk.PhotoImage(image_miniature)
    preview_canvas.create_image(frame_width / 2, frame_height / 2, image=tk_miniature)
    preview_canvas.tk_miniature = tk_miniature  # reference to avoid garbage collection
    return image_miniature


def update_previews(input_1, menu_var, input_preview_canvas, output_preview):
    """Update the input and output preview canvases from interface data."""

    try:
        # Open image
        filename = input_1.get()
        input_image = Image.open(filename)
        input_miniature = update_preview(input_image, input_preview_canvas)
        output_miniature = jk.convert_image(input_miniature, colormap=menu_var.get())
        update_preview(output_miniature, output_preview)

    except AttributeError as e:
        print("Error 1:", e, file=sys.stderr)
    except IOError as e:
        print("Error 2:", e, file=sys.stderr)


def browse_click(input_1, cm_menu, input_preview, output_preview):
    """Action when clicking on the browse button."""
    file = fd.askopenfilename()
    if file != ():
        input_1.delete(0, last=tk.END)
        input_1.insert(0, file)
        update_previews(input_1, cm_menu, input_preview, output_preview)


def save_click(input_1, v):
    """Action when clicking on the save button."""
    filename = fd.asksaveasfilename()
    try:
        jk.convert_file(input_1.get(), filename, colormap=v.get())
    except AttributeError as e:
        print("Error 3:", e, file=sys.stderr)
    except ValueError as e:
        print("Error 4:", e, file=sys.stderr)


def main():
    root = tk.Tk()
    root.title("Jet Killer")
    root.config(padx=30)
    root.config(pady=5)

    # Parameters zone
    parameters = tk.LabelFrame(root, text="Parameters")
    parameters.grid(column=0, row=0, sticky=tk.W + tk.E + tk.N + tk.S, pady=5)
    parameters.config(padx=10)
    parameters.config(pady=10)

    # Preview zone
    preview = tk.LabelFrame(root, text="Preview")
    preview.grid(column=0, row=1, sticky=tk.W + tk.E + tk.N + tk.S, pady=5)
    preview.config(padx=10)
    preview.config(pady=10)

    # Save zone
    save = tk.Frame(root)
    save.grid(column=0, row=2, sticky=tk.E, pady=5)
    save.config()

    # Preview of input file
    input_preview_label = tk.Label(preview, text="Input")
    input_preview_label.grid(column=0, row=0)
    input_preview_canvas = tk.Canvas(preview, width=400, height=200)
    input_preview_canvas.grid(column=0, row=1)

    # Preview of output file
    output_preview_label = tk.Label(preview, text="Output")
    output_preview_label.grid(column=1, row=0)
    output_preview_canvas = tk.Canvas(preview, width=400, height=200)
    output_preview_canvas.grid(column=1, row=1)

    # Input file entry
    input_file_prompt = tk.Label(parameters, text="Input file:")
    input_file_prompt.grid(column=0, row=0, sticky=tk.W)
    input_file_entry = tk.Entry(parameters, width=75)
    input_file_entry.grid(column=1, row=0)
    input_file_browse_button = tk.Button(parameters, text="...")
    input_file_browse_button.grid(column=2, row=0)

    # Colormap menu
    colormap_prompt = tk.Label(parameters, text="Colormap:")
    colormap_prompt.grid(column=0, row=1, sticky=tk.W)
    colormap_var = tk.StringVar()
    colormap_options = ("viridis", "plasma", "inferno", "magma")
    colormap_var.set("viridis")
    colormap_menu = tk.OptionMenu(parameters, colormap_var, *colormap_options)
    colormap_menu.grid(column=1, row=1, columnspan=2, sticky=tk.W)

    # Save button
    save_button = tk.Button(save, text="Convert and save")
    save_button.grid(column=0, row=0)

    # Quit button
    quit_button = tk.Button(save, text="Quit")
    quit_button.grid(column=1, row=0)

    # Actions
    def input_file_browse_button_action():
        return browse_click(input_file_entry, colormap_var, input_preview_canvas, output_preview_canvas)

    def save_button_action():
        return save_click(input_file_entry, colormap_var)

    def colormap_action(*args):
        return update_previews(input_file_entry, colormap_var, input_preview_canvas, output_preview_canvas)

    input_file_browse_button["command"] = input_file_browse_button_action
    save_button["command"] = save_button_action
    quit_button["command"] = root.quit
    colormap_var.trace("w", colormap_action)

    # Main loop
    root.mainloop()
